@@ -37,7 +37,7 @@ class BaseDatabaseCreation:
         If prefix is provided, it will be prepended to the database name
         to isolate it from other test databases.
         """
-        from plain.models.cli import migrate
+        from plain.models.cli.migrations import apply
 
         test_database_name = self._get_test_db_name(prefix)
 
@@ -52,20 +52,16 @@ class BaseDatabaseCreation:
         settings.DATABASE["NAME"] = test_database_name
         self.connection.settings_dict["NAME"] = test_database_name
 
-        # We report migrate messages at one level lower than that
-        # requested. This ensures we don't get flooded with messages during
-        # testing (unless you really ask to be flooded).
-        migrate.callback(
+        apply.callback(
             package_label=None,
             migration_name=None,
             fake=False,
             plan=False,
             check_unapplied=False,
             backup=False,
-            prune=False,
             no_input=True,
-            verbosity=max(verbosity - 1, 0),
             atomic_batch=False,  # No need for atomic batch when creating test database
+            quiet=verbosity < 2,  # Show migration output when verbosity is 2+
         )
 
         # Ensure a connection for the side effect of initializing the test database.
