@@ -1,10 +1,13 @@
+from __future__ import annotations
+
 import sys
+from typing import Any
 
 from plain.models.backends.base.creation import BaseDatabaseCreation
 
 
 class DatabaseCreation(BaseDatabaseCreation):
-    def sql_table_creation_suffix(self):
+    def sql_table_creation_suffix(self) -> str:
         suffix = []
         test_settings = self.connection.settings_dict["TEST"]
         if test_settings["CHARSET"]:
@@ -13,7 +16,7 @@ class DatabaseCreation(BaseDatabaseCreation):
             suffix.append("COLLATE {}".format(test_settings["COLLATION"]))
         return " ".join(suffix)
 
-    def _execute_create_test_db(self, cursor, parameters):
+    def _execute_create_test_db(self, cursor: Any, parameters: dict[str, Any]) -> None:
         try:
             super()._execute_create_test_db(cursor, parameters)
         except Exception as e:
