@@ -3,11 +3,12 @@ from __future__ import annotations
 import datetime
 import decimal
 import uuid
-from collections.abc import Callable
+from collections.abc import Callable, Iterable
 from functools import cached_property, lru_cache
 from typing import TYPE_CHECKING, Any
 
 from plain import models
+from plain.models.aggregates import Aggregate, Avg, StdDev, Sum, Variance
 from plain.models.backends.base.operations import BaseDatabaseOperations
 from plain.models.constants import OnConflict
 from plain.models.db import DatabaseError, NotSupportedError
@@ -18,9 +19,14 @@ from plain.utils.dateparse import parse_date, parse_datetime, parse_time
 
 if TYPE_CHECKING:
     from plain.models.backends.base.base import BaseDatabaseWrapper
+    from plain.models.backends.sqlite3.base import SQLiteDatabaseWrapper
+    from plain.models.fields import Field
 
 
 class DatabaseOperations(BaseDatabaseOperations):
+    # Type checker hint: connection is always SQLiteDatabaseWrapper in this class
+    connection: SQLiteDatabaseWrapper
+
     cast_char_field_without_max_length = "text"
     cast_data_types = {
         "DateField": "TEXT",
@@ -48,7 +54,7 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def check_expression_support(self, expression: Any) -> None:
         bad_fields = (models.DateField, models.DateTimeField, models.TimeField)
-        bad_aggregates = (models.Sum, models.Avg, models.Variance, models.StdDev)
+        bad_aggregates = (Sum, Avg, Variance, StdDev)
         if isinstance(expression, bad_aggregates):
             for expr in expression.get_source_expressions():
                 try:
@@ -65,7 +71,7 @@ class DatabaseOperations(BaseDatabaseOperations):
                             "since date/time is saved as text."
                         )
         if (
-            isinstance(expression, models.Aggregate)
+            isinstance(expression, Aggregate)
             and expression.distinct
             and len(expression.source_expressions) > 1
         ):
@@ -446,10 +452,10 @@ class DatabaseOperations(BaseDatabaseOperations):
 
     def on_conflict_suffix_sql(
         self,
-        fields: list[Any],
+        fields: list[Field],
         on_conflict: Any,
-        update_fields: list[Any],
-        unique_fields: list[Any],
+        update_fields: Iterable[str],
+        unique_fields: Iterable[str],
     ) -> str:
         if (
             on_conflict == OnConflict.UPDATE
