@@ -4,7 +4,7 @@ import copy
 import warnings
 from collections.abc import Iterable, Iterator, Sequence
 from itertools import chain
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, ClassVar, dataclass_transform
 
 if TYPE_CHECKING:
     from plain.models.meta import Meta
@@ -12,7 +12,7 @@ if TYPE_CHECKING:
 
 import plain.runtime
 from plain.exceptions import NON_FIELD_ERRORS, ValidationError
-from plain.models import models_registry, transaction
+from plain.models import models_registry, transaction, types
 from plain.models.constants import LOOKUP_SEP
 from plain.models.constraints import CheckConstraint, UniqueConstraint
 from plain.models.db import (
@@ -27,7 +27,7 @@ from plain.models.exceptions import (
     MultipleObjectsReturnedDescriptor,
 )
 from plain.models.expressions import RawSQL, Value
-from plain.models.fields import NOT_PROVIDED, PrimaryKeyField
+from plain.models.fields import NOT_PROVIDED
 from plain.models.fields.reverse_related import ForeignObjectRel
 from plain.models.meta import Meta
 from plain.models.options import Options
@@ -48,6 +48,7 @@ class Deferred:
 DEFERRED = Deferred()
 
 
+@dataclass_transform(kw_only_default=True)
 class ModelBase(type):
     """Metaclass for all models."""
 
@@ -91,16 +92,16 @@ class ModelState:
 
 class Model(metaclass=ModelBase):
     # Every model gets an automatic id field
-    id = PrimaryKeyField()
+    id: int = types.PrimaryKeyField()
 
     # Descriptors for other model behavior
-    query = QuerySet()
+    query: ClassVar[QuerySet[Model]] = QuerySet()
     model_options = Options()
     _model_meta = Meta()
     DoesNotExist = DoesNotExistDescriptor()
     MultipleObjectsReturned = MultipleObjectsReturnedDescriptor()
 
-    def __init__(self, *args: Any, **kwargs: Any):
+    def __init__(self, **kwargs: Any):
         # Alias some things as locals to avoid repeat global lookups
         cls = self.__class__
         meta = cls._model_meta
@@ -110,68 +111,32 @@ class Model(metaclass=ModelBase):
         # Set up the storage for instance state
         self._state = ModelState()
 
-        # There is a rather weird disparity here; if kwargs, it's set, then args
-        # overrides it. It should be one or the other; don't duplicate the work
-        # The reason for the kwargs check is that standard iterator passes in by
-        # args, and instantiation for iteration is 33% faster.
-        if len(args) > len(meta.concrete_fields):
-            # Daft, but matches old exception sans the err msg.
-            raise IndexError("Number of args exceeds number of fields")
-
-        if not kwargs:
-            fields_iter = iter(meta.concrete_fields)
-            # The ordering of the zip calls matter - zip throws StopIteration
-            # when an iter throws it. So if the first iter throws it, the second
-            # is *not* consumed. We rely on this, so don't change the order
-            # without changing the logic.
-            for val, field in zip(args, fields_iter):
-                if val is _DEFERRED:
-                    continue
-                _setattr(self, field.attname, val)
-        else:
-            # Slower, kwargs-ready version.
-            fields_iter = iter(meta.fields)
-            for val, field in zip(args, fields_iter):
-                if val is _DEFERRED:
-                    continue
-                _setattr(self, field.attname, val)
-                if kwargs.pop(field.name, NOT_PROVIDED) is not NOT_PROVIDED:
-                    raise TypeError(
-                        f"{cls.__qualname__}() got both positional and "
-                        f"keyword arguments for field '{field.name}'."
-                    )
-
-        # Now we're left with the unprocessed fields that *must* come from
-        # keywords, or default.
-
-        for field in fields_iter:
+        # Process all fields from kwargs or use defaults
+        for field in meta.fields:
             is_related_object = False
             # Virtual field
             if field.attname not in kwargs and field.column is None:
                 continue
-            if kwargs:
-                if isinstance(field.remote_field, ForeignObjectRel):
-                    try:
-                        # Assume object instance was passed in.
-                        rel_obj = kwargs.pop(field.name)
-                        is_related_object = True
-                    except KeyError:
-                        try:
-                            # Object instance wasn't passed in -- must be an ID.
-                            val = kwargs.pop(field.attname)
-                        except KeyError:
-                            val = field.get_default()
-                else:
+            if isinstance(field.remote_field, ForeignObjectRel):
+                try:
+                    # Assume object instance was passed in.
+                    rel_obj = kwargs.pop(field.name)
+                    is_related_object = True
+                except KeyError:
                     try:
+                        # Object instance wasn't passed in -- must be an ID.
                         val = kwargs.pop(field.attname)
                     except KeyError:
-                        # This is done with an exception rather than the
-                        # default argument on pop because we don't want
-                        # get_default() to be evaluated, and then not used.
-                        # Refs #12057.
                         val = field.get_default()
             else:
-                val = field.get_default()
+                try:
+                    val = kwargs.pop(field.attname)
+                except KeyError:
+                    # This is done with an exception rather than the
+                    # default argument on pop because we don't want
+                    # get_default() to be evaluated, and then not used.
+                    # Refs #12057.
+                    val = field.get_default()
 
             if is_related_object:
                 # If we are passed a related instance, set it using the
@@ -184,29 +149,29 @@ class Model(metaclass=ModelBase):
                 if val is not _DEFERRED:
                     _setattr(self, field.attname, val)
 
-        if kwargs:
-            property_names = meta._property_names
-            unexpected = ()
-            for prop, value in kwargs.items():
-                # Any remaining kwargs must correspond to properties or virtual
-                # fields.
-                if prop in property_names:
+        # Handle any remaining kwargs (properties or virtual fields)
+        property_names = meta._property_names
+        unexpected = ()
+        for prop, value in kwargs.items():
+            # Any remaining kwargs must correspond to properties or virtual
+            # fields.
+            if prop in property_names:
+                if value is not _DEFERRED:
+                    _setattr(self, prop, value)
+            else:
+                try:
+                    meta.get_field(prop)
+                except FieldDoesNotExist:
+                    unexpected += (prop,)
+                else:
                     if value is not _DEFERRED:
                         _setattr(self, prop, value)
-                else:
-                    try:
-                        meta.get_field(prop)
-                    except FieldDoesNotExist:
-                        unexpected += (prop,)
-                    else:
-                        if value is not _DEFERRED:
-                            _setattr(self, prop, value)
-            if unexpected:
-                unexpected_names = ", ".join(repr(n) for n in unexpected)
-                raise TypeError(
-                    f"{cls.__name__}() got unexpected keyword arguments: "
-                    f"{unexpected_names}"
-                )
+        if unexpected:
+            unexpected_names = ", ".join(repr(n) for n in unexpected)
+            raise TypeError(
+                f"{cls.__name__}() got unexpected keyword arguments: {unexpected_names}"
+            )
+
         super().__init__()
 
     @classmethod
@@ -217,7 +182,11 @@ class Model(metaclass=ModelBase):
                 next(values_iter) if f.attname in field_names else DEFERRED
                 for f in cls._model_meta.concrete_fields
             ]
-        new = cls(*values)
+        # Build kwargs dict from field names and values
+        field_dict = dict(
+            zip((f.attname for f in cls._model_meta.concrete_fields), values)
+        )
+        new = cls(**field_dict)
         new._state.adding = False
         return new
 
@@ -523,7 +492,7 @@ class Model(metaclass=ModelBase):
                 )
                 for f in non_pks
             ]
-            forced_update = update_fields or force_update
+            forced_update = bool(update_fields or force_update)
             updated = self._do_update(
                 base_qs, id_val, values, update_fields, forced_update
             )
@@ -689,7 +658,7 @@ class Model(metaclass=ModelBase):
 
     def _get_unique_checks(
         self, exclude: set[str] | None = None
-    ) -> list[tuple[type, tuple[str, ...]]]:
+    ) -> list[tuple[type[Model], tuple[str, ...]]]:
         """
         Return a list of checks to perform. Since validate_unique() could be
         called from a ModelForm, some fields may have been excluded; we can't
@@ -717,7 +686,7 @@ class Model(metaclass=ModelBase):
         return unique_checks
 
     def _perform_unique_checks(
-        self, unique_checks: list[tuple[type, tuple[str, ...]]]
+        self, unique_checks: list[tuple[type[Model], tuple[str, ...]]]
     ) -> dict[str, list[ValidationError]]:
         errors = {}
 
@@ -805,7 +774,7 @@ class Model(metaclass=ModelBase):
             )
 
     def get_constraints(self) -> list[tuple[type, list[Any]]]:
-        constraints = [(self.__class__, self.model_options.constraints)]
+        constraints = [(self.__class__, list(self.model_options.constraints))]
         return constraints
 
     def validate_constraints(self, exclude: set[str] | None = None) -> None:
@@ -1220,7 +1189,7 @@ class Model(metaclass=ModelBase):
     def _check_local_fields(
         cls, fields: Iterable[str], option: str
     ) -> list[PreflightResult]:
-        from plain import models
+        from plain.models.fields.reverse_related import ManyToManyRel
 
         # In order to avoid hitting the relation tree prematurely, we use our
         # own fields_map instead of using get_field()
@@ -1243,7 +1212,7 @@ class Model(metaclass=ModelBase):
                     )
                 )
             else:
-                if isinstance(field.remote_field, models.ManyToManyRel):
+                if isinstance(field.remote_field, ManyToManyRel):
                     errors.append(
                         PreflightResult(
                             fix=f"'{option}' refers to a ManyToManyField '{field_name}', but "
