@@ -1,3 +1,7 @@
+from __future__ import annotations
+
+from typing import Any
+
 from plain import signals
 
 from .connections import DatabaseConnection
@@ -22,7 +26,7 @@ db_connection = DatabaseConnection()
 
 
 # Register an event to reset saved queries when a Plain request is started.
-def reset_queries(**kwargs):
+def reset_queries(**kwargs: Any) -> None:
     if db_connection.has_connection():
         db_connection.queries_log.clear()
 
@@ -32,7 +36,7 @@ signals.request_started.connect(reset_queries)
 
 # Register an event to reset transaction state and close connections past
 # their lifetime.
-def close_old_connections(**kwargs):
+def close_old_connections(**kwargs: Any) -> None:
     if db_connection.has_connection():
         db_connection.close_if_unusable_or_obsolete()
 
