@@ -11,13 +11,13 @@ import warnings
 from base64 import b64decode, b64encode
 from collections.abc import Callable, Sequence
 from functools import cached_property, total_ordering
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, Generic, Protocol, TypeVar, overload
 
 from plain import exceptions, validators
 from plain.models.constants import LOOKUP_SEP
 from plain.models.db import db_connection
 from plain.models.enums import ChoicesMeta
-from plain.models.query_utils import DeferredAttribute, RegisterLookupMixin
+from plain.models.query_utils import RegisterLookupMixin
 from plain.preflight import PreflightResult
 from plain.utils import timezone
 from plain.utils.datastructures import DictWrapper
@@ -111,8 +111,12 @@ def return_None() -> None:
     return None
 
 
+# TypeVar for Field's generic type parameter
+T = TypeVar("T")
+
+
 @total_ordering
-class Field(RegisterLookupMixin):
+class Field(RegisterLookupMixin, Generic[T]):
     """Base class for all field types"""
 
     # Designates whether empty strings fundamentally are allowed at the
@@ -123,8 +127,8 @@ class Field(RegisterLookupMixin):
     # These track each time a Field instance is created. Used to retain order.
     # The auto_creation_counter is used for fields that Plain implicitly
     # creates, creation_counter is used for all user-specified fields.
-    creation_counter = 0
-    auto_creation_counter = -1
+    creation_counter: int = 0
+    auto_creation_counter: int = -1
     default_validators = []  # Default set of validators
     default_error_messages = {
         "invalid_choice": "Value %(value)r is not a valid choice.",
@@ -143,21 +147,16 @@ class Field(RegisterLookupMixin):
         "limit_choices_to",
         # Database-level options are not supported, see #21961.
         "on_delete",
-        "related_name",
         "related_query_name",
         "validators",
     )
 
     # Field flags
-    hidden = False
-
     many_to_many = None
     many_to_one = None
     one_to_many = None
     related_model = None
 
-    descriptor_class = DeferredAttribute
-
     # Generic field type description, usually overridden by subclasses
     def _description(self) -> str:
         return f"Field of type: {self.__class__.__name__}"
@@ -238,6 +237,7 @@ class Field(RegisterLookupMixin):
         Check if field name is valid, i.e. 1) does not end with an
         underscore, 2) does not contain "__" and 3) is not "id".
         """
+        assert self.name is not None, "Field name must be set before checking"
         if self.name.endswith("_"):
             return [
                 PreflightResult(
@@ -497,7 +497,7 @@ class Field(RegisterLookupMixin):
         elif path.startswith("plain.models.fields"):
             path = path.replace("plain.models.fields", "plain.models")
         # Return basic info - other fields should override this.
-        return (self.name, path, [], keywords)
+        return (self.name, path, [], keywords)  # type: ignore[return-value]
 
     def clone(self) -> Field:
         """
@@ -518,25 +518,37 @@ class Field(RegisterLookupMixin):
     def __lt__(self, other: object) -> bool:
         # This is needed because bisect does not take a comparison function.
         # Order by creation_counter first for backward compatibility.
-        if isinstance(other, Field):
-            if (
-                self.creation_counter != other.creation_counter
-                or not hasattr(self, "model")
-                and not hasattr(other, "model")
-            ):
-                return self.creation_counter < other.creation_counter
-            elif hasattr(self, "model") != hasattr(other, "model"):
-                return not hasattr(self, "model")  # Order no-model fields first
-            else:
-                # creation_counter's are equal, compare only models.
+        if not isinstance(other, Field):
+            return NotImplemented
+
+        # Type narrowing: other is now known to be a Field
+        other_field: Field[Any] = other
+
+        if (
+            self.creation_counter != other_field.creation_counter
+            or not hasattr(self, "model")
+            and not hasattr(other_field, "model")
+        ):
+            return self.creation_counter < other_field.creation_counter
+        elif hasattr(self, "model") != hasattr(other_field, "model"):
+            return not hasattr(self, "model")  # Order no-model fields first
+        else:
+            # creation_counter's are equal, compare only models.
+            # Use getattr with defaults to satisfy type checker
+            self_pkg = getattr(getattr(self, "model", None), "model_options", None)
+            other_pkg = getattr(
+                getattr(other_field, "model", None), "model_options", None
+            )
+            if self_pkg is not None and other_pkg is not None:
                 return (
-                    self.model.model_options.package_label,
-                    self.model.model_options.model_name,
+                    self_pkg.package_label,
+                    self_pkg.model_name,
                 ) < (
-                    other.model.model_options.package_label,
-                    other.model.model_options.model_name,
+                    other_pkg.package_label,
+                    other_pkg.model_name,
                 )
-        return NotImplemented
+            # Fallback if model_options not available
+            return self.creation_counter < other_field.creation_counter
 
     def __hash__(self) -> int:
         return hash(self.creation_counter)
@@ -571,7 +583,8 @@ class Field(RegisterLookupMixin):
         not a new copy of that field. So, use the app registry to load the
         model and then the field back.
         """
-        if not hasattr(self, "model"):
+        model = getattr(self, "model", None)
+        if model is None:
             # Fields are sometimes used without attaching them to models (for
             # example in aggregation). In this case give back a plain field
             # instance. The code below will create a new empty instance of
@@ -582,9 +595,11 @@ class Field(RegisterLookupMixin):
             # usage.
             state.pop("_get_default", None)
             return _empty, (self.__class__,), state
+        assert self.name is not None
+        options = model.model_options
         return _load_field, (
-            self.model.model_options.package_label,
-            self.model.model_options.object_name,
+            options.package_label,
+            options.object_name,
             self.name,
         )
 
@@ -763,8 +778,8 @@ class Field(RegisterLookupMixin):
     def get_db_converters(
         self, connection: BaseDatabaseWrapper
     ) -> list[Callable[..., Any]]:
-        if hasattr(self, "from_db_value"):
-            return [self.from_db_value]
+        if from_db_value := getattr(self, "from_db_value", None):
+            return [from_db_value]
         return []
 
     @property
@@ -783,14 +798,91 @@ class Field(RegisterLookupMixin):
     def contribute_to_class(self, cls: Any, name: str) -> None:
         """
         Register the field with the model class it belongs to.
+
+        Field now acts as its own descriptor - it stays on the class and handles
+        __get__/__set__/__delete__ directly.
         """
         self.set_attributes_from_name(name)
         self.model = cls
         cls._model_meta.add_field(self)
+
+        # Field is now a descriptor itself - ensure it's set on the class
+        # This is important for inherited fields that get deepcopied in Meta.__get__
         if self.column:
-            setattr(cls, self.attname, self.descriptor_class(self))
+            setattr(cls, self.attname, self)
+
+    # Descriptor protocol implementation
+    @overload
+    def __get__(self, instance: None, owner: type) -> Field[T]: ...
+
+    @overload
+    def __get__(self, instance: Any, owner: type) -> T: ...
+
+    def __get__(self, instance: Any | None, owner: type) -> Field[T] | T:
+        """
+        Descriptor __get__ for attribute access.
+
+        Class access (User.email) returns the Field descriptor itself.
+        Instance access (user.email) returns the field value from instance.__dict__,
+        with lazy loading support if the value is not yet loaded.
+        """
+        # Class access - return the Field descriptor
+        if instance is None:
+            return self
+
+        # If field hasn't been contributed to a class yet (e.g., used standalone
+        # as an output_field in aggregates), just return self
+        if not hasattr(self, "attname"):
+            return self
+
+        # Instance access - get value from instance dict
+        data = instance.__dict__
+        field_name = self.attname
+
+        # If value not in dict, lazy load from database
+        if field_name not in data:
+            # Deferred field - load it from the database
+            instance.refresh_from_db(fields=[field_name])
+
+        return data.get(field_name)
+
+    def __set__(self, instance: Any, value: Any) -> None:
+        """
+        Descriptor __set__ for attribute assignment.
+
+        Validates and converts the value using to_python(), then stores it
+        in instance.__dict__[attname].
+        """
+        # Safety check: ensure field has been properly initialized
+        if not hasattr(self, "attname"):
+            raise AttributeError(
+                f"Field {self.__class__.__name__} has not been initialized properly. "
+                f"The field's contribute_to_class() has not been called yet. "
+                f"This usually means the field is being used before it was added to a model class."
+            )
+
+        # Convert/validate the value
+        if value is not None:
+            value = self.to_python(value)
+
+        # Store in instance dict
+        instance.__dict__[self.attname] = value
+
+    def __delete__(self, instance: Any) -> None:
+        """
+        Descriptor __delete__ for attribute deletion.
+
+        Removes the value from instance.__dict__.
+        """
+        try:
+            del instance.__dict__[self.attname]
+        except KeyError:
+            raise AttributeError(
+                f"{instance.__class__.__name__!r} object has no attribute {self.attname!r}"
+            )
 
     def get_attname(self) -> str:
+        assert self.name is not None  # Field name must be set
         return self.name
 
     def get_attname_column(self) -> tuple[str, str]:
@@ -848,6 +940,15 @@ class Field(RegisterLookupMixin):
             return return_None
         return str  # return empty string
 
+    def get_limit_choices_to(self) -> Any:
+        """
+        Return ``limit_choices_to`` for this model field.
+        Overridden by related fields (ForeignKey, etc.).
+        """
+        raise NotImplementedError(
+            "get_limit_choices_to() should only be called on related fields"
+        )
+
     def get_choices(
         self,
         include_blank: bool = True,
@@ -868,13 +969,17 @@ class Field(RegisterLookupMixin):
                 if not blank_defined:
                     choices = blank_choice + choices
             return choices
-        rel_model = self.remote_field.model
+        remote_field = getattr(self, "remote_field", None)
+        if remote_field is None or getattr(remote_field, "model", None) is None:
+            return blank_choice if include_blank else []
+        rel_model = remote_field.model
         limit_choices_to = limit_choices_to or self.get_limit_choices_to()
-        choice_func = operator.attrgetter(
-            self.remote_field.get_related_field().attname
-            if hasattr(self.remote_field, "get_related_field")
+        related_field_name = (
+            remote_field.get_related_field().attname
+            if hasattr(remote_field, "get_related_field")
             else "id"
         )
+        choice_func = operator.attrgetter(related_field_name)
         qs = rel_model.query.complex_filter(limit_choices_to)
         if ordering:
             qs = qs.order_by(*ordering)
@@ -904,6 +1009,7 @@ class Field(RegisterLookupMixin):
     flatchoices = property(_get_flatchoices)
 
     def save_form_data(self, instance: Any, data: Any) -> None:
+        assert self.name is not None
         setattr(instance, self.name, data)
 
     def value_from_object(self, obj: Any) -> Any:
@@ -911,11 +1017,11 @@ class Field(RegisterLookupMixin):
         return getattr(obj, self.attname)
 
 
-class BooleanField(Field):
+class BooleanField(Field[bool]):
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value must be either True or False.",
-        "invalid_nullable": "“%(value)s” value must be either True, False, or None.",
+        "invalid": '"%(value)s" value must be either True or False.',
+        "invalid_nullable": '"%(value)s" value must be either True, False, or None.',
     }
     description = "Boolean (Either True or False)"
 
@@ -945,7 +1051,7 @@ class BooleanField(Field):
         return self.to_python(value)
 
 
-class CharField(Field):
+class CharField(Field[str]):
     def __init__(self, *, db_collation: str | None = None, **kwargs: Any):
         super().__init__(**kwargs)
         self.db_collation = db_collation
@@ -1053,10 +1159,10 @@ def _get_naive_now() -> datetime.datetime:
     return _to_naive(timezone.now())
 
 
-class DateTimeCheckMixin:
-    def preflight(self, **kwargs: Any) -> list[PreflightResult]:  # type: ignore[misc]
+class DateTimeCheckMixin(Field):  # type: ignore[type-arg]
+    def preflight(self, **kwargs: Any) -> list[PreflightResult]:
         return [
-            *super().preflight(**kwargs),  # type: ignore[misc]
+            *super().preflight(**kwargs),
             *self._check_mutually_exclusive_options(),
             *self._check_fix_default_value(),
         ]
@@ -1129,11 +1235,11 @@ class DateTimeCheckMixin:
         return []
 
 
-class DateField(DateTimeCheckMixin, Field):
+class DateField(DateTimeCheckMixin, Field[datetime.date]):
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value has an invalid date format. It must be in YYYY-MM-DD format.",
-        "invalid_date": "“%(value)s” value has the correct format (YYYY-MM-DD) but it is an invalid date.",
+        "invalid": '"%(value)s" value has an invalid date format. It must be in YYYY-MM-DD format.',
+        "invalid_date": '"%(value)s" value has the correct format (YYYY-MM-DD) but it is an invalid date.',
     }
     description = "Date (without time)"
 
@@ -1235,9 +1341,9 @@ class DateField(DateTimeCheckMixin, Field):
 class DateTimeField(DateField):
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.",
-        "invalid_date": "“%(value)s” value has the correct format (YYYY-MM-DD) but it is an invalid date.",
-        "invalid_datetime": "“%(value)s” value has the correct format (YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ]) but it is an invalid date/time.",
+        "invalid": '"%(value)s" value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.',
+        "invalid_date": '"%(value)s" value has the correct format (YYYY-MM-DD) but it is an invalid date.',
+        "invalid_datetime": '"%(value)s" value has the correct format (YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ]) but it is an invalid date/time.',
     }
     description = "Date (with time)"
 
@@ -1351,10 +1457,10 @@ class DateTimeField(DateField):
         return "" if val is None else val.isoformat()
 
 
-class DecimalField(Field):
+class DecimalField(Field[decimal.Decimal]):
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value must be a decimal number.",
+        "invalid": '"%(value)s" value must be a decimal number.',
     }
     description = "Decimal number"
 
@@ -1499,7 +1605,7 @@ class DecimalField(Field):
         return self.to_python(value)
 
 
-class DurationField(Field):
+class DurationField(Field[datetime.timedelta]):
     """
     Store timedelta objects.
 
@@ -1509,7 +1615,7 @@ class DurationField(Field):
 
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value has an invalid format. It must be in [DD] [[HH:]MM:]ss[.uuuuuu] format.",
+        "invalid": '"%(value)s" value has an invalid format. It must be in [DD] [[HH:]MM:]ss[.uuuuuu] format.',
     }
     description = "Duration"
 
@@ -1573,10 +1679,10 @@ class EmailField(CharField):
         return name, path, args, kwargs
 
 
-class FloatField(Field):
+class FloatField(Field[float]):
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value must be a float.",
+        "invalid": '"%(value)s" value must be a float.',
     }
     description = "Floating point number"
 
@@ -1607,10 +1713,10 @@ class FloatField(Field):
             )
 
 
-class IntegerField(Field):
+class IntegerField(Field[int]):
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value must be an integer.",
+        "invalid": '"%(value)s" value must be an integer.',
     }
     description = "Integer"
 
@@ -1714,7 +1820,7 @@ class SmallIntegerField(IntegerField):
         return "SmallIntegerField"
 
 
-class GenericIPAddressField(Field):
+class GenericIPAddressField(Field[str]):
     empty_strings_allowed = False
     description = "IP address"
     default_error_messages = {}
@@ -1800,7 +1906,17 @@ class GenericIPAddressField(Field):
         return str(value)
 
 
-class PositiveIntegerRelDbTypeMixin:
+class _HasDbType(Protocol):
+    """Protocol for objects that have a db_type method and integer_field_class."""
+
+    integer_field_class: type[IntegerField]
+
+    def db_type(self, connection: BaseDatabaseWrapper) -> str | None: ...
+
+
+class PositiveIntegerRelDbTypeMixin(IntegerField):
+    integer_field_class: type[IntegerField]
+
     def __init_subclass__(cls, **kwargs: Any) -> None:
         super().__init_subclass__(**kwargs)
         if not hasattr(cls, "integer_field_class"):
@@ -1810,10 +1926,10 @@ class PositiveIntegerRelDbTypeMixin:
                     for parent in cls.__mro__[1:]
                     if issubclass(parent, IntegerField)
                 ),
-                None,
+                None,  # type: ignore[arg-type]
             )
 
-    def rel_db_type(self, connection: BaseDatabaseWrapper) -> str | None:
+    def rel_db_type(self: _HasDbType, connection: BaseDatabaseWrapper) -> str | None:
         """
         Return the data type that a related field pointing to this field should
         use. In most cases, a foreign key pointing to a positive integer
@@ -1849,7 +1965,7 @@ class PositiveSmallIntegerField(PositiveIntegerRelDbTypeMixin, SmallIntegerField
         return "PositiveSmallIntegerField"
 
 
-class TextField(Field):
+class TextField(Field[str]):
     description = "Text"
 
     def __init__(self, *, db_collation: str | None = None, **kwargs: Any):
@@ -1904,11 +2020,11 @@ class TextField(Field):
         return name, path, args, kwargs
 
 
-class TimeField(DateTimeCheckMixin, Field):
+class TimeField(DateTimeCheckMixin, Field[datetime.time]):
     empty_strings_allowed = False
     default_error_messages = {
-        "invalid": "“%(value)s” value has an invalid format. It must be in HH:MM[:ss[.uuuuuu]] format.",
-        "invalid_time": "“%(value)s” value has the correct format (HH:MM[:ss[.uuuuuu]]) but it is an invalid time.",
+        "invalid": '"%(value)s" value has an invalid format. It must be in HH:MM[:ss[.uuuuuu]] format.',
+        "invalid_time": '"%(value)s" value has the correct format (HH:MM[:ss[.uuuuuu]]) but it is an invalid time.',
     }
     description = "Time"
 
@@ -2023,7 +2139,7 @@ class URLField(CharField):
         return name, path, args, kwargs
 
 
-class BinaryField(Field):
+class BinaryField(Field[bytes | memoryview]):
     description = "Raw binary data"
     empty_values = [None, b""]
 
@@ -2082,9 +2198,9 @@ class BinaryField(Field):
         return value
 
 
-class UUIDField(Field):
+class UUIDField(Field[uuid.UUID]):
     default_error_messages = {
-        "invalid": "“%(value)s” is not a valid UUID.",
+        "invalid": '"%(value)s" is not a valid UUID.',
     }
     description = "Universally unique identifier"
     empty_strings_allowed = False
@@ -2152,7 +2268,7 @@ class PrimaryKeyField(BigIntegerField):
 
     def deconstruct(self) -> tuple[str, str, list[Any], dict[str, Any]]:
         # PrimaryKeyField takes no parameters, so we return an empty kwargs dict
-        return (self.name, "plain.models.PrimaryKeyField", [], {})
+        return (self.name, "plain.models.PrimaryKeyField", [], {})  # type: ignore[return-value]
 
     def validate(self, value: Any, model_instance: Any) -> None:
         pass
