@@ -1,9 +1,9 @@
 """
 Accessors for related objects.
 
-When a field defines a relation between two models, each model class provides
-an attribute to access related instances of the other model class (unless the
-reverse accessor has been disabled with related_name='+').
+When a field defines a relation between two models, the forward model provides
+an attribute to access related instances. Reverse accessors must be explicitly
+defined using ReverseForeignKey or ReverseManyToMany descriptors.
 
 Accessors are implemented as descriptors in order to customize access and
 assignment. This module defines the descriptor classes.
@@ -12,38 +12,31 @@ Forward accessors follow foreign keys. Reverse accessors trace them back. For
 example, with the following models::
 
     class Parent(Model):
-        pass
+        children: ReverseForeignKey[Child] = ReverseForeignKey(to="Child", field="parent")
 
     class Child(Model):
-        parent = ForeignKey(Parent, related_name='children')
+        parent: Parent = ForeignKey(Parent, on_delete=models.CASCADE)
 
- ``child.parent`` is a forward many-to-one relation. ``parent.children`` is a
-reverse many-to-one relation.
+ ``child.parent`` is a forward foreign key relation. ``parent.children`` is a
+reverse foreign key relation.
 
-1. Related instance on the forward side of a many-to-one relation:
-   ``ForwardManyToOneDescriptor``.
+1. Related instance on the forward side of a foreign key relation:
+   ``ForwardForeignKeyDescriptor``.
 
    Uniqueness of foreign key values is irrelevant to accessing the related
    instance, making the many-to-one and one-to-one cases identical as far as
    the descriptor is concerned. The constraint is checked upstream (unicity
    validation in forms) or downstream (unique indexes in the database).
 
-2. Related objects manager for related instances on the reverse side of a
-   many-to-one relation: ``ReverseManyToOneDescriptor``.
-
-   Unlike the previous two classes, this one provides access to a collection
-   of objects. It returns a manager rather than an instance.
-
-3. Related objects manager for related instances on the forward or reverse
-   sides of a many-to-many relation: ``ManyToManyDescriptor``.
+2. Related objects manager for related instances on the forward or reverse
+   sides of a many-to-many relation: ``ForwardManyToManyDescriptor``.
 
    Many-to-many relations are symmetrical. The syntax of Plain models
    requires declaring them on one side but that's an implementation detail.
    They could be declared on the other side without any change in behavior.
-   Therefore the forward and reverse descriptors can be the same.
 
-   If you're looking for ``ForwardManyToManyDescriptor`` or
-   ``ReverseManyToManyDescriptor``, use ``ManyToManyDescriptor`` instead.
+Reverse relations must be explicitly defined using ``ReverseForeignKey`` or
+``ReverseManyToMany`` descriptors on the model class.
 """
 
 from __future__ import annotations
@@ -52,35 +45,21 @@ from functools import cached_property
 from typing import Any
 
 from plain.models.query import QuerySet
-from plain.models.query_utils import DeferredAttribute
 from plain.utils.functional import LazyObject
 
-from .related_managers import (
-    ForwardManyToManyManager,
-    ReverseManyToManyManager,
-    ReverseManyToOneManager,
-)
+from .related_managers import ManyToManyManager
 
 
-class ForeignKeyDeferredAttribute(DeferredAttribute):
-    def __set__(self, instance: Any, value: Any) -> None:
-        if instance.__dict__.get(self.field.attname) != value and self.field.is_cached(
-            instance
-        ):
-            self.field.delete_cached_value(instance)
-        instance.__dict__[self.field.attname] = value
-
-
-class ForwardManyToOneDescriptor:
+class ForwardForeignKeyDescriptor:
     """
-    Accessor to the related object on the forward side of a many-to-one relation.
+    Accessor to the related object on the forward side of a foreign key relation.
 
     In the example::
 
         class Child(Model):
-            parent = ForeignKey(Parent, related_name='children')
+            parent: Parent = ForeignKey(Parent, on_delete=models.CASCADE)
 
-    ``Child.parent`` is a ``ForwardManyToOneDescriptor`` instance.
+    ``Child.parent`` is a ``ForwardForeignKeyDescriptor`` instance.
     """
 
     def __init__(self, field_with_rel: Any) -> None:
@@ -120,12 +99,11 @@ class ForwardManyToOneDescriptor:
         remote_field = self.field.remote_field
 
         # FIXME: This will need to be revisited when we introduce support for
-        # composite fields. In the meantime we take this practical approach to
-        # solve a regression on 1.6 when the reverse manager in hidden
-        # (related_name ends with a '+'). Refs #21410.
+        # composite fields. In the meantime we take this practical approach.
+        # Refs #21410.
         # The check for len(...) == 1 is a special case that allows the query
         # to be join-less and smaller. Refs #21760.
-        if remote_field.is_hidden() or len(self.field.foreign_related_fields) == 1:
+        if len(self.field.foreign_related_fields) == 1:
             query = {
                 f"{related_field.name}__in": {
                     instance_attr(inst)[0] for inst in instances
@@ -157,7 +135,7 @@ class ForwardManyToOneDescriptor:
 
     def __get__(
         self, instance: Any | None, cls: type | None = None
-    ) -> ForwardManyToOneDescriptor | Any | None:
+    ) -> ForwardForeignKeyDescriptor | Any | None:
         """
         Get the related instance through the forward relation.
 
@@ -266,13 +244,17 @@ class ForwardManyToOneDescriptor:
         return getattr, (self.field.model, self.field.name)
 
 
-class RelationDescriptorBase:
+class ForwardManyToManyDescriptor:
     """
-    Base class for relation descriptors that don't allow direct assignment.
+    Accessor to the related objects manager on the forward side of a
+    many-to-many relation.
 
-    This is used for descriptors that manage collections of related objects
-    (reverse FK and M2M relations). Forward FK relations don't inherit from
-    this because they allow direct assignment.
+    In the example::
+
+        class Pizza(Model):
+            toppings: ManyToManyField[Topping] = ManyToManyField(Topping, through=PizzaTopping)
+
+    ``Pizza.toppings`` is a ``ForwardManyToManyDescriptor`` instance.
     """
 
     def __init__(self, rel: Any) -> None:
@@ -281,120 +263,28 @@ class RelationDescriptorBase:
 
     def __get__(
         self, instance: Any | None, cls: type | None = None
-    ) -> RelationDescriptorBase | Any:
-        """
-        Get the related manager when the descriptor is accessed.
-
-        Subclasses must implement get_related_manager().
-        """
+    ) -> ForwardManyToManyDescriptor | Any:
+        """Get the related manager when the descriptor is accessed."""
         if instance is None:
             return self
-        return self.get_related_manager(instance)
-
-    def get_related_manager(self, instance: Any) -> Any:
-        """Return the appropriate manager for this relation."""
-        raise NotImplementedError(
-            f"{self.__class__.__name__} must implement get_related_manager()"
-        )
-
-    def _get_set_deprecation_msg_params(self) -> tuple[str, str]:
-        """Return parameters for the error message when direct assignment is attempted."""
-        raise NotImplementedError(
-            f"{self.__class__.__name__} must implement _get_set_deprecation_msg_params()"
+        return ManyToManyManager(
+            instance=instance,
+            field=self.rel.field,
+            through=self.rel.through,
+            related_model=self.rel.model,
+            is_reverse=False,
+            symmetrical=self.rel.symmetrical,
         )
 
     def __set__(self, instance: Any, value: Any) -> None:
         """Prevent direct assignment to the relation."""
         raise TypeError(
-            "Direct assignment to the {} is prohibited. Use {}.set() instead.".format(
-                *self._get_set_deprecation_msg_params()
-            ),
+            f"Direct assignment to the forward side of a many-to-many set is prohibited. Use {self.field.name}.set() instead.",
         )
 
-
-class ReverseManyToOneDescriptor(RelationDescriptorBase):
-    """
-    Accessor to the related objects manager on the reverse side of a
-    many-to-one relation.
-
-    In the example::
-
-        class Child(Model):
-            parent = ForeignKey(Parent, related_name='children')
-
-    ``Parent.children`` is a ``ReverseManyToOneDescriptor`` instance.
-
-    Most of the implementation is delegated to the ReverseManyToOneManager class.
-    """
-
-    def get_related_manager(self, instance: Any) -> ReverseManyToOneManager:
-        """Return the ReverseManyToOneManager for this relation."""
-        return ReverseManyToOneManager(instance, self.rel)
-
-    def _get_set_deprecation_msg_params(self) -> tuple[str, str]:
-        return (
-            "reverse side of a related set",
-            self.rel.get_accessor_name(),
-        )
-
-
-class ForwardManyToManyDescriptor(RelationDescriptorBase):
-    """
-    Accessor to the related objects manager on the forward side of a
-    many-to-many relation.
-
-    In the example::
-
-        class Pizza(Model):
-            toppings = ManyToManyField(Topping, related_name='pizzas')
-
-    ``Pizza.toppings`` is a ``ForwardManyToManyDescriptor`` instance.
-    """
-
     @property
     def through(self) -> Any:
         # through is provided so that you have easy access to the through
         # model (Book.authors.through) for inlines, etc. This is done as
         # a property to ensure that the fully resolved value is returned.
         return self.rel.through
-
-    def get_related_manager(self, instance: Any) -> ForwardManyToManyManager:
-        """Return the ForwardManyToManyManager for this relation."""
-        return ForwardManyToManyManager(instance, self.rel)
-
-    def _get_set_deprecation_msg_params(self) -> tuple[str, str]:
-        return (
-            "forward side of a many-to-many set",
-            self.field.name,
-        )
-
-
-class ReverseManyToManyDescriptor(RelationDescriptorBase):
-    """
-    Accessor to the related objects manager on the reverse side of a
-    many-to-many relation.
-
-    In the example::
-
-        class Pizza(Model):
-            toppings = ManyToManyField(Topping, related_name='pizzas')
-
-    ``Topping.pizzas`` is a ``ReverseManyToManyDescriptor`` instance.
-    """
-
-    @property
-    def through(self) -> Any:
-        # through is provided so that you have easy access to the through
-        # model (Book.authors.through) for inlines, etc. This is done as
-        # a property to ensure that the fully resolved value is returned.
-        return self.rel.through
-
-    def get_related_manager(self, instance: Any) -> ReverseManyToManyManager:
-        """Return the ReverseManyToManyManager for this relation."""
-        return ReverseManyToManyManager(instance, self.rel)
-
-    def _get_set_deprecation_msg_params(self) -> tuple[str, str]:
-        return (
-            "reverse side of a many-to-many set",
-            self.rel.get_accessor_name(),
-        )
