@@ -7,6 +7,7 @@ through foreign key and many-to-many relationships.
 
 from __future__ import annotations
 
+from abc import ABC, abstractmethod
 from typing import Any
 
 from plain.models import transaction
@@ -41,7 +42,7 @@ def _filter_prefetch_queryset(
     return queryset.filter(predicate)
 
 
-class BaseRelatedManager:
+class BaseRelatedManager(ABC):
     """
     Base class for all related object managers.
 
@@ -53,24 +54,25 @@ class BaseRelatedManager:
         """Access the QuerySet for this relationship."""
         return self.get_queryset()
 
+    @abstractmethod
     def get_queryset(self) -> QuerySet:
         """Return the QuerySet for this relationship."""
-        raise NotImplementedError("Subclasses must implement get_queryset()")
+        ...
 
 
-class ReverseManyToOneManager(BaseRelatedManager):
+class ReverseForeignKeyManager(BaseRelatedManager):
     """
-    Manager for the reverse side of a many-to-one relation.
+    Manager for the reverse side of a foreign key relation.
 
-    This manager adds behaviors specific to many-to-one relations.
+    This manager adds behaviors specific to foreign key relations.
     """
 
-    def __init__(self, instance: Any, rel: Any):
-        self.model = rel.related_model
+    def __init__(self, instance: Any, field: Any, related_model: Any):
+        self.model = related_model
         self.instance = instance
-        self.field = rel.field
+        self.field = field
         self.core_filters = {self.field.name: instance}
-        self.allow_null = rel.field.allow_null
+        self.allow_null = self.field.allow_null
 
     def _check_fk_val(self) -> None:
         for field in self.field.foreign_related_fields:
@@ -278,22 +280,52 @@ class ReverseManyToOneManager(BaseRelatedManager):
             self.add(*objs, bulk=bulk)
 
 
-class BaseManyToManyManager(BaseRelatedManager):
+class ManyToManyManager(BaseRelatedManager):
     """
-    Base class for many-to-many managers with common functionality.
+    Manager for both forward and reverse sides of a many-to-many relation.
 
-    Subclasses must set these attributes in __init__:
-    - model
-    - query_field_name
-    - prefetch_cache_name
-    - source_field_name
-    - target_field_name
-    - symmetrical (for forward relations)
+    This manager handles both directions of many-to-many relations with
+    conditional logic for symmetrical relationships (which only apply to
+    forward relations).
     """
 
-    def __init__(self, instance: Any, rel: Any):
+    # Type hints for attributes
+    model: Any
+    query_field_name: str
+    prefetch_cache_name: str
+    source_field_name: str
+    target_field_name: str
+
+    def __init__(
+        self,
+        instance: Any,
+        field: Any,
+        through: Any,
+        related_model: Any,
+        is_reverse: bool,
+        symmetrical: bool = False,
+    ):
+        # Set direction-specific attributes
+        if is_reverse:
+            # Reverse: accessing from the target model back to the source
+            self.model = related_model
+            self.query_field_name = field.name
+            self.prefetch_cache_name = field.related_query_name()
+            self.source_field_name = field.m2m_reverse_field_name()
+            self.target_field_name = field.m2m_field_name()
+            self.symmetrical = False  # Reverse relations are never symmetrical
+        else:
+            # Forward: accessing from the source model to the target
+            self.model = related_model
+            self.query_field_name = field.related_query_name()
+            self.prefetch_cache_name = field.name
+            self.source_field_name = field.m2m_field_name()
+            self.target_field_name = field.m2m_reverse_field_name()
+            self.symmetrical = symmetrical
+
+        # Initialize common M2M attributes
         self.instance = instance
-        self.through = rel.through
+        self.through = through
 
         self.source_field = self.through._model_meta.get_field(self.source_field_name)
         self.target_field = self.through._model_meta.get_field(self.target_field_name)
@@ -533,35 +565,6 @@ class BaseManyToManyManager(BaseRelatedManager):
             filters = self._build_remove_filters(old_vals)
             self.through.query.filter(filters).delete()
 
-    # Subclasses must implement these methods:
-    def _build_remove_filters(self, removed_vals: Any) -> Any:
-        raise NotImplementedError
-
-    def add(self, *objs: Any, through_defaults: dict[str, Any] | None = None) -> None:
-        raise NotImplementedError
-
-    def remove(self, *objs: Any) -> None:
-        raise NotImplementedError
-
-
-class ForwardManyToManyManager(BaseManyToManyManager):
-    """
-    Manager for the forward side of a many-to-many relation.
-
-    This manager adds behaviors specific to many-to-many relations.
-    """
-
-    def __init__(self, instance: Any, rel: Any):
-        # Set required attributes before calling super().__init__
-        self.model = rel.model
-        self.query_field_name = rel.field.related_query_name()
-        self.prefetch_cache_name = rel.field.name
-        self.source_field_name = rel.field.m2m_field_name()
-        self.target_field_name = rel.field.m2m_reverse_field_name()
-        self.symmetrical = rel.symmetrical
-
-        super().__init__(instance, rel)
-
     def _build_remove_filters(self, removed_vals: Any) -> Any:
         filters = Q.create([(self.source_field_name, self.related_val)])
         # No need to add a subquery condition if removed_vals is a QuerySet without
@@ -573,6 +576,7 @@ class ForwardManyToManyManager(BaseManyToManyManager):
             filters = filters & Q.create(  # type: ignore[unsupported-operator]
                 [(f"{self.target_field_name}__in", removed_vals)]
             )
+        # Add symmetrical filters for forward symmetrical relations
         if self.symmetrical:
             symmetrical_filters = Q.create([(self.target_field_name, self.related_val)])
             if removed_vals_filters:
@@ -604,51 +608,3 @@ class ForwardManyToManyManager(BaseManyToManyManager):
     def remove(self, *objs: Any) -> None:
         self._remove_prefetched_objects()
         self._remove_items(self.source_field_name, self.target_field_name, *objs)
-
-
-class ReverseManyToManyManager(BaseManyToManyManager):
-    """
-    Manager for the reverse side of a many-to-many relation.
-
-    This manager adds behaviors specific to many-to-many relations.
-    """
-
-    def __init__(self, instance: Any, rel: Any):
-        # Set required attributes before calling super().__init__
-        self.model = rel.related_model
-        self.query_field_name = rel.field.name
-        self.prefetch_cache_name = rel.field.related_query_name()
-        self.source_field_name = rel.field.m2m_reverse_field_name()
-        self.target_field_name = rel.field.m2m_field_name()
-        self.symmetrical = False  # Reverse relations are never symmetrical
-
-        super().__init__(instance, rel)
-
-    def _build_remove_filters(self, removed_vals: Any) -> Any:
-        filters = Q.create([(self.source_field_name, self.related_val)])
-        # No need to add a subquery condition if removed_vals is a QuerySet without
-        # filters.
-        removed_vals_filters = (
-            not isinstance(removed_vals, QuerySet) or removed_vals._has_filters()
-        )
-        if removed_vals_filters:
-            filters = filters & Q.create(  # type: ignore[unsupported-operator]
-                [(f"{self.target_field_name}__in", removed_vals)]
-            )
-        # Note: reverse relations are never symmetrical, so no symmetrical logic here
-        return filters
-
-    def add(self, *objs: Any, through_defaults: dict[str, Any] | None = None) -> None:
-        self._remove_prefetched_objects()
-        with transaction.atomic(savepoint=False):
-            self._add_items(
-                self.source_field_name,
-                self.target_field_name,
-                *objs,
-                through_defaults=through_defaults,
-            )
-            # Reverse relations are never symmetrical, so no mirror entry logic
-
-    def remove(self, *objs: Any) -> None:
-        self._remove_prefetched_objects()
-        self._remove_items(self.source_field_name, self.target_field_name, *objs)
