@@ -12,7 +12,7 @@ they're the closest concept currently available.
 from __future__ import annotations
 
 from functools import cached_property
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 from plain.models.exceptions import FieldDoesNotExist, FieldError
 from plain.utils.hashable import make_hashable
@@ -20,6 +20,9 @@ from plain.utils.hashable import make_hashable
 from . import BLANK_CHOICE_DASH
 from .mixins import FieldCacheMixin
 
+if TYPE_CHECKING:
+    from plain.models.lookups import Lookup
+
 
 class ForeignObjectRel(FieldCacheMixin):
     """
@@ -43,14 +46,12 @@ class ForeignObjectRel(FieldCacheMixin):
         self,
         field: Any,
         to: Any,
-        related_name: str | None = None,
         related_query_name: str | None = None,
         limit_choices_to: Any = None,
         on_delete: Any = None,
     ):
         self.field = field
         self.model = to
-        self.related_name = related_name
         self.related_query_name = related_query_name
         self.limit_choices_to = {} if limit_choices_to is None else limit_choices_to
         self.on_delete = on_delete
@@ -62,10 +63,6 @@ class ForeignObjectRel(FieldCacheMixin):
     # __init__ as the field doesn't have its model yet. Calling these methods
     # before field.contribute_to_class() has been called will result in
     # AttributeError
-    @cached_property
-    def hidden(self) -> bool:
-        return self.is_hidden()
-
     @cached_property
     def name(self) -> str:
         return self.field.related_query_name()
@@ -106,7 +103,7 @@ class ForeignObjectRel(FieldCacheMixin):
     def one_to_many(self) -> bool:
         return self.field.many_to_one
 
-    def get_lookup(self, lookup_name: str) -> Any:
+    def get_lookup(self, lookup_name: str) -> type[Lookup] | None:
         return self.field.get_lookup(lookup_name)
 
     def get_internal_type(self) -> str:
@@ -124,7 +121,6 @@ class ForeignObjectRel(FieldCacheMixin):
         return (
             self.field,
             self.model,
-            self.related_name,
             self.related_query_name,
             make_hashable(self.limit_choices_to),
             self.on_delete,
@@ -144,7 +140,7 @@ class ForeignObjectRel(FieldCacheMixin):
         state = self.__dict__.copy()
         # Delete the path_infos cached property because it can be recalculated
         # at first invocation after deserialization. The attribute must be
-        # removed because subclasses like ManyToOneRel may have a PathInfo
+        # removed because subclasses like ForeignKeyRel may have a PathInfo
         # which contains an intermediate M2M table that's been dynamically
         # created and doesn't exist in the .models module.
         # This is a reverse relation, so there is no reverse_path_infos to
@@ -172,10 +168,6 @@ class ForeignObjectRel(FieldCacheMixin):
             qs = qs.order_by(*ordering)
         return (blank_choice if include_blank else []) + [(x.id, str(x)) for x in qs]
 
-    def is_hidden(self) -> bool:
-        """Should the related object be hidden?"""
-        return not self.related_name
-
     def get_joining_columns(self) -> Any:
         return self.field.get_reverse_joining_columns()
 
@@ -192,21 +184,6 @@ class ForeignObjectRel(FieldCacheMixin):
         # example custom multicolumn joins currently have no remote field).
         self.field_name = None
 
-    def get_accessor_name(self, model: Any = None) -> str | None:
-        # This method encapsulates the logic that decides what name to give an
-        # accessor descriptor that retrieves related many-to-one or
-        # many-to-many objects.
-        model = model or self.related_model
-        if self.multiple:
-            # If this is a symmetrical m2m relation on self, there is no
-            # reverse accessor.
-            if self.symmetrical and model == self.model:
-                return None
-        # Only return a name if related_name is explicitly set
-        if self.related_name:
-            return self.related_name
-        return None
-
     def get_path_info(self, filtered_relation: Any = None) -> Any:
         if filtered_relation:
             return self.field.get_reverse_path_info(filtered_relation)
@@ -221,11 +198,14 @@ class ForeignObjectRel(FieldCacheMixin):
         """
         Return the name of the cache key to use for storing an instance of the
         forward model on the reverse model.
+
+        Uses the related_query_name for caching, which provides a stable name
+        for prefetch_related operations.
         """
-        return self.get_accessor_name()
+        return self.field.related_query_name()
 
 
-class ManyToOneRel(ForeignObjectRel):
+class ForeignKeyRel(ForeignObjectRel):
     """
     Used by the ForeignKey field to store information about the relation.
 
@@ -234,9 +214,9 @@ class ManyToOneRel(ForeignObjectRel):
 
     Note: Because we somewhat abuse the Rel objects by using them as reverse
     fields we get the funny situation where
-    ``ManyToOneRel.many_to_one == False`` and
-    ``ManyToOneRel.one_to_many == True``. This is unfortunate but the actual
-    ManyToOneRel class is a private API and there is work underway to turn
+    ``ForeignKeyRel.many_to_one == False`` and
+    ``ForeignKeyRel.one_to_many == True``. This is unfortunate but the actual
+    ForeignKeyRel class is a private API and there is work underway to turn
     reverse relations into actual fields.
     """
 
@@ -244,7 +224,6 @@ class ManyToOneRel(ForeignObjectRel):
         self,
         field: Any,
         to: Any,
-        related_name: str | None = None,
         related_query_name: str | None = None,
         limit_choices_to: Any = None,
         on_delete: Any = None,
@@ -252,7 +231,6 @@ class ManyToOneRel(ForeignObjectRel):
         super().__init__(
             field,
             to,
-            related_name=related_name,
             related_query_name=related_query_name,
             limit_choices_to=limit_choices_to,
             on_delete=on_delete,
@@ -297,7 +275,6 @@ class ManyToManyRel(ForeignObjectRel):
         *,
         through: Any,
         through_fields: tuple[str, str] | None = None,
-        related_name: str | None = None,
         related_query_name: str | None = None,
         limit_choices_to: Any = None,
         symmetrical: bool = True,
@@ -305,7 +282,6 @@ class ManyToManyRel(ForeignObjectRel):
         super().__init__(
             field,
             to,
-            related_name=related_name,
             related_query_name=related_query_name,
             limit_choices_to=limit_choices_to,
         )
@@ -327,7 +303,7 @@ class ManyToManyRel(ForeignObjectRel):
     def get_related_field(self) -> Any:
         """
         Return the field in the 'to' object to which this relationship is tied.
-        Provided for symmetry with ManyToOneRel.
+        Provided for symmetry with ForeignKeyRel.
         """
         meta = self.through._model_meta
         if self.through_fields:
