@@ -168,12 +168,15 @@ class ModelFormMetaclass(DeclarativeFieldsMetaclass):
         bases: tuple[type, ...],
         attrs: dict[str, Any],
     ) -> type[BaseModelForm]:
-        new_class = super().__new__(mcs, name, bases, attrs)  # type: ignore[invalid-super-argument]
+        # Metaclass __new__ returns a type, specifically type[BaseModelForm]
+        new_class: type[BaseModelForm] = super().__new__(  # type: ignore[assignment]
+            mcs, name, bases, attrs
+        )
 
         if bases == (BaseModelForm,):
             return new_class
 
-        opts = new_class._meta = ModelFormOptions(getattr(new_class, "Meta", None))
+        opts = new_class._meta = ModelFormOptions(getattr(new_class, "Meta", None))  # type: ignore[unresolved-attribute]
 
         # We check if a string was passed to `fields`,
         # which is likely to be a mistake where the user typed ('foo') instead
@@ -206,18 +209,18 @@ class ModelFormMetaclass(DeclarativeFieldsMetaclass):
 
             # make sure opts.fields doesn't specify an invalid field
             none_model_fields = {k for k, v in fields.items() if not v}
-            missing_fields = none_model_fields.difference(new_class.declared_fields)
+            missing_fields = none_model_fields.difference(new_class.declared_fields)  # type: ignore[unresolved-attribute]
             if missing_fields:
                 message = "Unknown field(s) (%s) specified for %s"
                 message %= (", ".join(missing_fields), opts.model.__name__)
                 raise FieldError(message)
             # Override default model fields with any custom declared ones
             # (plus, include all the other declared fields).
-            fields.update(new_class.declared_fields)
+            fields.update(new_class.declared_fields)  # type: ignore[unresolved-attribute]
         else:
-            fields = new_class.declared_fields
+            fields = new_class.declared_fields  # type: ignore[unresolved-attribute]
 
-        new_class.base_fields = fields
+        new_class.base_fields = fields  # type: ignore[invalid-assignment]
 
         return new_class
 
@@ -232,7 +235,7 @@ class BaseModelForm(BaseForm):
         initial: dict[str, Any] | None = None,
         instance: Any = None,
     ) -> None:
-        opts = self._meta
+        opts = self._meta  # type: ignore[unresolved-attribute]
         if opts.model is None:
             raise ValueError("ModelForm has no model class specified.")
         if instance is None:
@@ -274,12 +277,12 @@ class BaseModelForm(BaseForm):
             # Don't perform model validation on fields that were defined
             # manually on the form and excluded via the ModelForm's Meta
             # class. See #12901.
-            elif self._meta.fields and field not in self._meta.fields:
+            elif self._meta.fields and field not in self._meta.fields:  # type: ignore[unresolved-attribute]
                 exclude.add(f.name)
 
             # Exclude fields that failed form validation. There's no need for
             # the model fields to validate them as well.
-            elif field in self._errors:
+            elif field in self._errors:  # type: ignore[unsupported-operator]
                 exclude.add(f.name)
 
             # Exclude empty fields that are not required by the form, if the
@@ -306,7 +309,7 @@ class BaseModelForm(BaseForm):
     def _update_errors(self, errors: ValidationError) -> None:
         # Override any validation error messages defined at the model level
         # with those defined at the form level.
-        opts = self._meta
+        opts = self._meta  # type: ignore[unresolved-attribute]
 
         # Allow the model generated by construct_instance() to raise
         # ValidationError and have them handled in the same way as others.
@@ -337,7 +340,7 @@ class BaseModelForm(BaseForm):
         self.add_error(None, errors)
 
     def _post_clean(self) -> None:
-        opts = self._meta
+        opts = self._meta  # type: ignore[unresolved-attribute]
 
         exclude = self._get_validation_exclusions()
 
@@ -371,7 +374,7 @@ class BaseModelForm(BaseForm):
         Save the many-to-many fields and generic relations for this form.
         """
         cleaned_data = self.cleaned_data
-        fields = self._meta.fields
+        fields = self._meta.fields  # type: ignore[unresolved-attribute]
         meta = self.instance._model_meta
 
         for f in meta.many_to_many:
@@ -498,7 +501,7 @@ class ModelChoiceField(ChoiceField):
         result = super(ChoiceField, self).__deepcopy__(memo)
         # Need to force a new ModelChoiceIterator to be created, bug #11183
         if self.queryset is not None:
-            result.queryset = self.queryset.all()
+            result.queryset = self.queryset.all()  # type: ignore[unresolved-attribute]
         return result
 
     def _get_queryset(self) -> Any:
@@ -513,7 +516,9 @@ class ModelChoiceField(ChoiceField):
         # If self._choices is set, then somebody must have manually set
         # the property self.choices. In this case, just return self._choices.
         if hasattr(self, "_choices"):
-            return self._choices
+            # After checking hasattr, we know _choices exists and is ModelChoiceIterator
+            choices: ModelChoiceIterator = self._choices  # type: ignore[assignment]
+            return choices
 
         # Otherwise, execute the QuerySet in self.queryset to determine the
         # choices dynamically. Return a fresh ModelChoiceIterator that has not been
@@ -657,7 +662,7 @@ def modelfield_to_formfield(
     choices_form_class: type[Field] | None = None,
     **kwargs: Any,
 ) -> Field | None:
-    defaults = {
+    defaults: dict[str, Any] = {
         "required": modelfield.required,
     }
 
