@@ -4,6 +4,8 @@ from collections.abc import Callable
 from contextlib import nullcontext
 from typing import TYPE_CHECKING, Any
 
+from plain.models.connections import DatabaseConnection
+
 from ..transaction import atomic
 from .loader import MigrationLoader
 from .migration import Migration
@@ -22,7 +24,7 @@ class MigrationExecutor:
 
     def __init__(
         self,
-        connection: BaseDatabaseWrapper,
+        connection: BaseDatabaseWrapper | DatabaseConnection,
         progress_callback: Callable[..., Any] | None = None,
     ) -> None:
         self.connection = connection
@@ -40,7 +42,8 @@ class MigrationExecutor:
         if clean_start:
             applied = {}
         else:
-            applied = dict(self.loader.applied_migrations)
+            applied_source = self.loader.applied_migrations or {}
+            applied = dict(applied_source)
         for target in targets:
             for migration in self.loader.graph.forwards_plan(target):
                 if migration not in applied:
@@ -61,9 +64,10 @@ class MigrationExecutor:
             full_plan = self.migration_plan(
                 self.loader.graph.leaf_nodes(), clean_start=True
             )
+            applied_source = self.loader.applied_migrations or {}
             applied_migrations = {
                 self.loader.graph.nodes[key]
-                for key in self.loader.applied_migrations
+                for key in applied_source
                 if key in self.loader.graph.nodes
             }
             for migration in full_plan:
@@ -126,11 +130,7 @@ class MigrationExecutor:
                         break
                     if migration in migrations_to_run:
                         if "models_registry" not in state.__dict__:
-                            if self.progress_callback:
-                                self.progress_callback("render_start")
                             state.models_registry  # Render all -- performance critical
-                            if self.progress_callback:
-                                self.progress_callback("render_success")
                         state = self.apply_migration(state, migration, fake=fake)
                         migrations_to_run.remove(migration)
 
@@ -145,13 +145,15 @@ class MigrationExecutor:
         """Run a migration forwards."""
         migration_recorded = False
         if self.progress_callback:
-            self.progress_callback("apply_start", migration, fake)
+            self.progress_callback("apply_start", migration=migration, fake=fake)
         if not fake:
             # Alright, do it normally
             with self.connection.schema_editor(
                 atomic=migration.atomic
             ) as schema_editor:
-                state = migration.apply(state, schema_editor)
+                state = migration.apply(
+                    state, schema_editor, operation_callback=self.progress_callback
+                )
                 if not schema_editor.deferred_sql:
                     self.record_migration(migration)
                     migration_recorded = True
@@ -159,7 +161,7 @@ class MigrationExecutor:
             self.record_migration(migration)
         # Report progress
         if self.progress_callback:
-            self.progress_callback("apply_success", migration, fake)
+            self.progress_callback("apply_success", migration=migration, fake=fake)
         return state
 
     def record_migration(self, migration: Migration) -> None:
