@@ -5,6 +5,7 @@ import sys
 from importlib import import_module, reload
 from typing import TYPE_CHECKING, Any
 
+from plain.models.connections import DatabaseConnection
 from plain.models.migrations.graph import MigrationGraph
 from plain.models.migrations.recorder import MigrationRecorder
 from plain.packages import packages_registry
@@ -50,7 +51,7 @@ class MigrationLoader:
 
     def __init__(
         self,
-        connection: BaseDatabaseWrapper | None,
+        connection: BaseDatabaseWrapper | DatabaseConnection | None,
         load: bool = True,
         ignore_no_migrations: bool = False,
         replace_migrations: bool = True,
@@ -76,7 +77,7 @@ class MigrationLoader:
         """
 
         # This package (plain-models) has different code under migrations/
-        if package_label == "models":
+        if package_label == "plainmodels":
             return None, True
 
         app = packages_registry.get_package_config(package_label)
@@ -162,6 +163,7 @@ class MigrationLoader:
         Return the migration(s) which match the given app label and name_prefix.
         """
         # Do the search
+        assert self.disk_migrations is not None, "load_disk() must be called first"
         results = []
         for migration_package_label, migration_name in self.disk_migrations:
             if migration_package_label == package_label and migration_name.startswith(
@@ -220,7 +222,8 @@ class MigrationLoader:
         for parent in migration.dependencies:
             # Ignore __first__ references to the same app.
             if parent[0] == key[0] and parent[1] != "__first__":
-                self.graph.add_dependency(migration, key, parent, skip_validation=True)
+                # Migration object is used only for error messages in add_dependency
+                self.graph.add_dependency(migration, key, parent, skip_validation=True)  # type: ignore[arg-type]
 
     def add_external_dependencies(
         self, key: tuple[str, str], migration: Migration
@@ -231,7 +234,8 @@ class MigrationLoader:
                 continue
             parent = self.check_key(parent, key[0])
             if parent is not None:
-                self.graph.add_dependency(migration, key, parent, skip_validation=True)
+                # Migration object is used only for error messages in add_dependency
+                self.graph.add_dependency(migration, key, parent, skip_validation=True)  # type: ignore[arg-type]
 
     def build_graph(self) -> None:
         """
@@ -241,6 +245,7 @@ class MigrationLoader:
         """
         # Load disk data
         self.load_disk()
+        assert self.disk_migrations is not None  # load_disk() ensures this
         # Load database data
         if self.connection is None:
             self.applied_migrations = {}
@@ -316,7 +321,9 @@ class MigrationLoader:
             raise
         self.graph.ensure_not_cyclic()
 
-    def check_consistent_history(self, connection: BaseDatabaseWrapper) -> None:
+    def check_consistent_history(
+        self, connection: BaseDatabaseWrapper | DatabaseConnection
+    ) -> None:
         """
         Raise InconsistentMigrationHistory if any applied migrations have
         unapplied dependencies.
@@ -331,9 +338,10 @@ class MigrationLoader:
                 if parent not in applied:
                     # Skip unapplied squashed migrations that have all of their
                     # `replaces` applied.
-                    if parent in self.replacements:
+                    # Use parent.key for dict lookup (Node.__eq__ allows `in` check)
+                    if parent.key in self.replacements:
                         if all(
-                            m in applied for m in self.replacements[parent].replaces
+                            m in applied for m in self.replacements[parent.key].replaces
                         ):
                             continue
                     raise InconsistentMigrationHistory(
@@ -370,23 +378,3 @@ class MigrationLoader:
         return self.graph.make_state(
             nodes=nodes, at_end=at_end, real_packages=self.unmigrated_packages
         )
-
-    def collect_sql(self, plan: list[Migration]) -> list[str]:
-        """
-        Take a migration plan and return a list of collected SQL statements
-        that represent the best-efforts version of that plan.
-        """
-        statements = []
-        state = None
-        for migration in plan:
-            with self.connection.schema_editor(
-                collect_sql=True, atomic=migration.atomic
-            ) as schema_editor:
-                if state is None:
-                    state = self.project_state(
-                        (migration.package_label, migration.name), at_end=False
-                    )
-
-                state = migration.apply(state, schema_editor, collect_sql=True)
-            statements.extend(schema_editor.collected_sql)
-        return statements
