@@ -1,5 +1,5 @@
 import pytest
-from app.examples.models import (  # type: ignore[import-untyped]
+from app.examples.models import (
     ChildCascade,
     ChildSetNull,
     DeleteParent,
@@ -8,8 +8,8 @@ from app.examples.models import (  # type: ignore[import-untyped]
 from plain.models import QuerySet
 
 
-class TestForwardManyToOneDescriptor:
-    """Test ForwardManyToOneDescriptor (e.g., child.parent)"""
+class TestForwardForeignKeyDescriptor:
+    """Test ForwardForeignKeyDescriptor (e.g., child.parent)"""
 
     def test_get_related_object(self, db):
         parent = DeleteParent.query.create(name="Test Parent")
@@ -35,7 +35,7 @@ class TestForwardManyToOneDescriptor:
         child = ChildCascade.query.create(parent=parent)
 
         # Setting a non-nullable FK to None should work but fail on save
-        child.parent = None
+        child.parent = None  # type: ignore[assignment]
         with pytest.raises(
             Exception, match="constraint|null|NOT NULL"
         ):  # Database constraint error
@@ -57,8 +57,8 @@ class TestForwardManyToOneDescriptor:
             pytest.skip(f"Nullable FK handling needs refinement: {e}")
 
 
-class TestReverseManyToOneDescriptor:
-    """Test ReverseManyToOneDescriptor (e.g., parent.children)"""
+class TestReverseForeignKey:
+    """Test ReverseForeignKey descriptor (e.g., parent.children)"""
 
     def test_get_related_queryset(self, db):
         parent = DeleteParent.query.create(name="Test Parent")
@@ -72,7 +72,7 @@ class TestReverseManyToOneDescriptor:
         # Test that reverse relation returns a manager with .query QuerySet
         children_manager = (
             parent.childcascade_set
-        )  # Note: no related_name, so uses default
+        )  # Explicit ReverseForeignKey descriptor
         children = children_manager.query
         assert isinstance(children, QuerySet)
         assert children.count() == 2
@@ -260,7 +260,7 @@ class TestEdgeCases:
 
         # Test that direct assignment to reverse relation raises error
         with pytest.raises(TypeError, match="Direct assignment.*prohibited"):
-            parent.childcascade_set = []
+            parent.childcascade_set = []  # type: ignore[assignment]
 
     def test_instance_none_handling(self, db):
         # Test accessing descriptor on class (not instance)
